@@ -1,6 +1,6 @@
 use derive_more::{Display, From};
 use ntex::util::Either;
-use std::io;
+use std::{error, io};
 
 /// Errors which can occur when attempting to handle mqtt connection.
 #[derive(Debug)]
@@ -13,8 +13,8 @@ pub enum MqttError<E> {
     HandshakeTimeout,
     /// Peer disconnect
     Disconnected,
-    /// Protocol specific unhandled error (for v3.1.1 only)
-    V3ProtocolError,
+    /// Server error
+    ServerError(&'static str),
 }
 
 /// Protocol level errors
@@ -49,6 +49,8 @@ pub enum ProtocolError {
     Io(io::Error),
 }
 
+impl error::Error for ProtocolError {}
+
 impl<E> From<ProtocolError> for MqttError<E> {
     fn from(err: ProtocolError) -> Self {
         MqttError::Protocol(err)
@@ -98,6 +100,8 @@ pub enum DecodeError {
     Utf8Error(std::str::Utf8Error),
 }
 
+impl error::Error for DecodeError {}
+
 #[derive(Copy, Clone, Debug, Display, PartialEq, Eq, Hash)]
 pub enum EncodeError {
     InvalidLength,
@@ -106,6 +110,8 @@ pub enum EncodeError {
     UnsupportedVersion,
 }
 
+impl error::Error for EncodeError {}
+
 impl PartialEq for DecodeError {
     fn eq(&self, other: &Self) -> bool {
         match (self, other) {
@@ -138,3 +144,5 @@ pub enum SendPacketError {
     #[display(fmt = "Peer disconnected")]
     Disconnected,
 }
+
+impl error::Error for SendPacketError {}
