@@ -1,8 +1,8 @@
 use std::{fmt, future::Future, marker, pin::Pin, rc::Rc, task::Context, task::Poll, time};
 
 use ntex::codec::{AsyncRead, AsyncWrite};
-use ntex::rt::time::{sleep, Sleep};
 use ntex::service::{apply_fn_factory, boxed, IntoServiceFactory, Service, ServiceFactory};
+use ntex::time::{sleep, Seconds, Sleep};
 use ntex::util::{timeout::Timeout, timeout::TimeoutError, Either, Ready};
 
 use crate::error::{MqttError, ProtocolError};
@@ -14,22 +14,18 @@ use super::handshake::{Handshake, HandshakeAck};
 use super::shared::{MqttShared, MqttSinkPool};
 use super::{codec as mqtt, dispatcher::factory, MqttServer, MqttSink, Publish, Session};
 
-pub(crate) type SelectItem<Io> =
-    (mqtt::Connect, Io, State, Rc<MqttShared>, Option<Pin<Box<Sleep>>>);
+pub(crate) type SelectItem<Io> = (Handshake<Io>, State, Option<Sleep>);
 
 type ServerFactory<Io, Err, InitErr> = boxed::BoxServiceFactory<
     (),
-    (mqtt::Connect, Io, State, Rc<MqttShared>, Option<Pin<Box<Sleep>>>),
+    SelectItem<Io>,
     Either<SelectItem<Io>, ()>,
     MqttError<Err>,
     InitErr,
 >;
 
-type Server<Io, Err> = boxed::BoxService<
-    (mqtt::Connect, Io, State, Rc<MqttShared>, Option<Pin<Box<Sleep>>>),
-    Either<SelectItem<Io>, ()>,
-    MqttError<Err>,
->;
+type Server<Io, Err> =
+    boxed::BoxService<SelectItem<Io>, Either<SelectItem<Io>, ()>, MqttError<Err>>;
 
 /// Mqtt server selector
 ///
@@ -38,7 +34,7 @@ type Server<Io, Err> = boxed::BoxService<
 pub struct Selector<Io, Err, InitErr> {
     servers: Vec<ServerFactory<Io, Err, InitErr>>,
     max_size: u32,
-    handshake_timeout: u16,
+    handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
     _t: marker::PhantomData<(Io, Err, InitErr)>,
 }
@@ -49,7 +45,7 @@ impl<Io, Err, InitErr> Selector<Io, Err, InitErr> {
         Selector {
             servers: Vec::new(),
             max_size: 0,
-            handshake_timeout: 0,
+            handshake_timeout: Seconds::ZERO,
             pool: Default::default(),
             _t: marker::PhantomData,
         }
@@ -62,11 +58,11 @@ where
     Err: 'static,
     InitErr: 'static,
 {
-    /// Set handshake timeout in millis.
+    /// Set handshake timeout.
     ///
     /// Handshake includes `connect` packet and response `connect-ack`.
     /// By default handshake timeuot is disabled.
-    pub fn handshake_timeout(mut self, timeout: u16) -> Self {
+    pub fn handshake_timeout(mut self, timeout: Seconds) -> Self {
         self.handshake_timeout = timeout;
         self
     }
@@ -87,7 +83,7 @@ where
         mut server: MqttServer<Io, St, C, Cn, P>,
     ) -> Self
     where
-        F: Fn(&mqtt::Connect) -> R + 'static,
+        F: Fn(&Handshake<Io>) -> R + 'static,
         R: Future<Output = Result<bool, Err>> + 'static,
         St: 'static,
         C: ServiceFactory<
@@ -99,7 +95,7 @@ where
             > + 'static,
         Cn: ServiceFactory<
                 Config = Session<St>,
-                Request = ControlMessage,
+                Request = ControlMessage<C::Error>,
                 Response = ControlResult,
             > + 'static,
         P: ServiceFactory<Config = Session<St>, Request = Publish, Response = ()> + 'static,
@@ -119,7 +115,7 @@ where
         self,
     ) -> impl ServiceFactory<
         Config = (),
-        Request = (Io, State, Option<Pin<Box<Sleep>>>),
+        Request = (Io, State, Option<Sleep>),
         Response = (),
         Error = MqttError<Err>,
         InitError = InitErr,
@@ -166,7 +162,7 @@ where
 pub struct SelectorService<Io, Err> {
     servers: Rc<Vec<Server<Io, Err>>>,
     max_size: u32,
-    handshake_timeout: u16,
+    handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 }
 
@@ -216,11 +212,7 @@ where
             16,
             self.pool.clone(),
         ));
-        let delay = if self.handshake_timeout > 0 {
-            Some(Box::pin(sleep(time::Duration::from_secs(self.handshake_timeout as u64))))
-        } else {
-            None
-        };
+        let delay = self.handshake_timeout.map(sleep);
 
         Box::pin(async move {
             // read first packet
@@ -250,7 +242,7 @@ where
             };
 
             // call servers
-            let mut item = (connect, io, state, shared, delay);
+            let mut item = (Handshake::new(connect, io, shared), state, delay);
             for srv in servers.iter() {
                 match srv.call(item).await? {
                     Either::Left(result) => {
@@ -279,7 +271,7 @@ where
     InitErr: 'static,
 {
     type Config = ();
-    type Request = (Io, State, Option<Pin<Box<Sleep>>>);
+    type Request = (Io, State, Option<Sleep>);
     type Response = ();
     type Error = MqttError<Err>;
     type InitError = InitErr;
@@ -312,7 +304,7 @@ where
     Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
 {
-    type Request = (Io, State, Option<Pin<Box<Sleep>>>);
+    type Request = (Io, State, Option<Sleep>);
     type Response = ();
     type Error = MqttError<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<(), MqttError<Err>>>>>;
@@ -381,7 +373,7 @@ where
             };
 
             // call servers
-            let mut item = (connect, io, state, shared, delay);
+            let mut item = (Handshake::new(connect, io, shared), state, delay);
             for srv in servers.iter() {
                 match srv.call(item).await? {
                     Either::Left(result) => {
@@ -390,7 +382,7 @@ where
                     Either::Right(_) => return Ok(()),
                 }
             }
-            log::error!("Cannot handle CONNECT packet {:?}", item.0);
+            log::error!("Cannot handle CONNECT packet {:?}", item.0.packet());
             Err(MqttError::ServerError("Cannot handle CONNECT packet"))
         })
     }
