@@ -1,9 +1,9 @@
 use std::task::{Context, Poll};
-use std::{fmt, future::Future, marker::PhantomData, pin::Pin, rc::Rc, time::Duration};
+use std::{fmt, future::Future, marker::PhantomData, pin::Pin, rc::Rc};
 
 use ntex::codec::{AsyncRead, AsyncWrite, Decoder, Encoder};
-use ntex::rt::time::Sleep;
 use ntex::service::{apply_fn_factory, IntoServiceFactory, Service, ServiceFactory};
+use ntex::time::{Millis, Seconds, Sleep};
 use ntex::util::{timeout::Timeout, timeout::TimeoutError, Either, Ready};
 
 use crate::error::{MqttError, ProtocolError};
@@ -24,8 +24,8 @@ pub struct MqttServer<Io, St, C: ServiceFactory, Cn: ServiceFactory, P: ServiceF
     publish: P,
     max_size: u32,
     inflight: usize,
-    handshake_timeout: u16,
-    disconnect_timeout: u16,
+    handshake_timeout: Seconds,
+    disconnect_timeout: Seconds,
     pub(super) pool: Rc<MqttSinkPool>,
     _t: PhantomData<(Io, St)>,
 }
@@ -55,8 +55,8 @@ where
             publish: DefaultPublishService::default(),
             max_size: 0,
             inflight: 16,
-            handshake_timeout: 0,
-            disconnect_timeout: 3000,
+            handshake_timeout: Seconds::ZERO,
+            disconnect_timeout: Seconds(3),
             pool: Default::default(),
             _t: PhantomData,
         }
@@ -69,8 +69,11 @@ where
     St: 'static,
     C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
         + 'static,
-    Cn: ServiceFactory<Config = Session<St>, Request = ControlMessage, Response = ControlResult>
-        + 'static,
+    Cn: ServiceFactory<
+            Config = Session<St>,
+            Request = ControlMessage<C::Error>,
+            Response = ControlResult,
+        > + 'static,
     P: ServiceFactory<Config = Session<St>, Request = Publish, Response = ()> + 'static,
 
     C::Error: From<Cn::Error>
@@ -79,16 +82,16 @@ where
         + From<P::InitError>
         + fmt::Debug,
 {
-    /// Set handshake timeout in millis.
+    /// Set handshake timeout.
     ///
     /// Handshake includes `connect` packet and response `connect-ack`.
     /// By default handshake timeuot is disabled.
-    pub fn handshake_timeout(mut self, timeout: u16) -> Self {
+    pub fn handshake_timeout(mut self, timeout: Seconds) -> Self {
         self.handshake_timeout = timeout;
         self
     }
 
-    /// Set server connection disconnect timeout in milliseconds.
+    /// Set server connection disconnect timeout.
     ///
     /// Defines a timeout for disconnect connection. If a disconnect procedure does not complete
     /// within this time, the connection get dropped.
@@ -96,7 +99,7 @@ where
     /// To disable timeout set value to 0.
     ///
     /// By default disconnect timeout is set to 3 seconds.
-    pub fn disconnect_timeout(mut self, val: u16) -> Self {
+    pub fn disconnect_timeout(mut self, val: Seconds) -> Self {
         self.disconnect_timeout = val;
         self
     }
@@ -127,7 +130,7 @@ where
         F: IntoServiceFactory<Srv>,
         Srv: ServiceFactory<
                 Config = Session<St>,
-                Request = ControlMessage,
+                Request = ControlMessage<C::Error>,
                 Response = ControlResult,
             > + 'static,
         C::Error: From<Srv::Error> + From<Srv::InitError>,
@@ -174,12 +177,10 @@ where
         let publish = self
             .publish
             .into_factory()
-            .map_err(|e| MqttError::Service(e.into()))
-            .map_init_err(|e| MqttError::Service(e.into()));
-        let control = self
-            .control
-            .map_err(|e| MqttError::Service(e.into()))
+            .map_err(|e| e.into())
             .map_init_err(|e| MqttError::Service(e.into()));
+        let control =
+            self.control.map_err(|e| e.into()).map_init_err(|e| MqttError::Service(e.into()));
 
         FramedService::new(
             handshake_service_factory(
@@ -188,26 +189,7 @@ where
                 self.handshake_timeout,
                 self.pool,
             ),
-            apply_fn_factory(
-                factory(publish, control, self.inflight),
-                |req: DispatchItem<Rc<MqttShared>>, srv| match req {
-                    DispatchItem::Item(req) => Either::Left(srv.call(req)),
-                    DispatchItem::KeepAliveTimeout => Either::Right(Ready::Err(
-                        MqttError::Protocol(ProtocolError::KeepAliveTimeout),
-                    )),
-                    DispatchItem::EncoderError(e) => {
-                        Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Encode(e))))
-                    }
-                    DispatchItem::DecoderError(e) => {
-                        Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Decode(e))))
-                    }
-                    DispatchItem::IoError(e) => {
-                        Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Io(e))))
-                    }
-                    DispatchItem::WBackPressureEnabled
-                    | DispatchItem::WBackPressureDisabled => Either::Right(Ready::Ok(None)),
-                },
-            ),
+            factory(publish, control, self.inflight),
             self.disconnect_timeout,
         )
     }
@@ -217,7 +199,7 @@ where
         self,
     ) -> impl ServiceFactory<
         Config = (),
-        Request = (Io, State, Option<Pin<Box<Sleep>>>),
+        Request = (Io, State, Option<Sleep>),
         Response = (),
         Error = MqttError<C::Error>,
         InitError = C::InitError,
@@ -226,12 +208,10 @@ where
         let publish = self
             .publish
             .into_factory()
-            .map_err(|e| MqttError::Service(e.into()))
-            .map_init_err(|e| MqttError::Service(e.into()));
-        let control = self
-            .control
-            .map_err(|e| MqttError::Service(e.into()))
+            .map_err(|e| e.into())
             .map_init_err(|e| MqttError::Service(e.into()));
+        let control =
+            self.control.map_err(|e| e.into()).map_init_err(|e| MqttError::Service(e.into()));
 
         FramedService2::new(
             handshake_service_factory2(
@@ -240,26 +220,7 @@ where
                 self.handshake_timeout,
                 self.pool,
             ),
-            apply_fn_factory(
-                factory(publish, control, self.inflight),
-                |req: DispatchItem<Rc<MqttShared>>, srv| match req {
-                    DispatchItem::Item(req) => Either::Left(srv.call(req)),
-                    DispatchItem::KeepAliveTimeout => Either::Right(Ready::Err(
-                        MqttError::Protocol(ProtocolError::KeepAliveTimeout),
-                    )),
-                    DispatchItem::EncoderError(e) => {
-                        Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Encode(e))))
-                    }
-                    DispatchItem::DecoderError(e) => {
-                        Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Decode(e))))
-                    }
-                    DispatchItem::IoError(e) => {
-                        Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Io(e))))
-                    }
-                    DispatchItem::WBackPressureEnabled
-                    | DispatchItem::WBackPressureDisabled => Either::Right(Ready::Ok(None)),
-                },
-            ),
+            factory(publish, control, self.inflight),
             self.disconnect_timeout,
         )
     }
@@ -276,47 +237,21 @@ where
         InitError = C::InitError,
     >
     where
-        F: Fn(&mqtt::Connect) -> R + 'static,
+        F: Fn(&Handshake<Io>) -> R + 'static,
         R: Future<Output = Result<bool, C::Error>> + 'static,
     {
-        let publish = self
-            .publish
-            .map_err(|e| MqttError::Service(e.into()))
-            .map_init_err(|e| MqttError::Service(e.into()));
-        let control = self
-            .control
-            .map_err(|e| MqttError::Service(e.into()))
-            .map_init_err(|e| MqttError::Service(e.into()));
-
-        let handler = apply_fn_factory(
-            factory(publish, control, self.inflight),
-            |req: DispatchItem<Rc<MqttShared>>, srv| match req {
-                DispatchItem::Item(req) => Either::Left(srv.call(req)),
-                DispatchItem::KeepAliveTimeout => Either::Right(Ready::Err(
-                    MqttError::Protocol(ProtocolError::KeepAliveTimeout),
-                )),
-                DispatchItem::EncoderError(e) => {
-                    Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Encode(e))))
-                }
-                DispatchItem::DecoderError(e) => {
-                    Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Decode(e))))
-                }
-                DispatchItem::IoError(e) => {
-                    Either::Right(Ready::Err(MqttError::Protocol(ProtocolError::Io(e))))
-                }
-                DispatchItem::WBackPressureEnabled | DispatchItem::WBackPressureDisabled => {
-                    Either::Right(Ready::Ok(None))
-                }
-            },
-        );
+        let publish =
+            self.publish.map_err(|e| e.into()).map_init_err(|e| MqttError::Service(e.into()));
+        let control =
+            self.control.map_err(|e| e.into()).map_init_err(|e| MqttError::Service(e.into()));
 
         ServerSelector {
             check: Rc::new(check),
             connect: self.handshake,
-            handler: Rc::new(handler),
+            handler: Rc::new(factory(publish, control, self.inflight)),
             max_size: self.max_size,
             disconnect_timeout: self.disconnect_timeout,
-            time: Timer::with(Duration::from_secs(1)),
+            time: Timer::new(Millis::ONE_SEC),
             _t: PhantomData,
         }
     }
@@ -325,12 +260,12 @@ where
 fn handshake_service_factory<Io, St, C>(
     factory: C,
     max_size: u32,
-    handshake_timeout: u16,
+    handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 ) -> impl ServiceFactory<
     Config = (),
     Request = Io,
-    Response = (Io, State, Rc<MqttShared>, Session<St>, u16),
+    Response = (Io, State, Rc<MqttShared>, Session<St>, Seconds),
     Error = MqttError<C::Error>,
 >
 where
@@ -338,18 +273,21 @@ where
     C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>,
     C::Error: fmt::Debug,
 {
-    ntex::apply(
-        Timeout::new(Duration::from_millis(handshake_timeout as u64)),
-        ntex::fn_factory(move || {
+    ntex::service::apply(
+        Timeout::new(Millis::from(handshake_timeout)),
+        ntex::service::fn_factory(move || {
             let pool = pool.clone();
             let fut = factory.new_service(());
             async move {
                 let service = fut.await?;
                 let pool = pool.clone();
                 let service = Rc::new(service.map_err(MqttError::Service));
-                Ok::<_, C::InitError>(ntex::apply_fn(service, move |conn: Io, service| {
-                    handshake(conn, None, service.clone(), max_size, pool.clone())
-                }))
+                Ok::<_, C::InitError>(ntex::service::apply_fn(
+                    service,
+                    move |conn: Io, service| {
+                        handshake(conn, None, service.clone(), max_size, pool.clone())
+                    },
+                ))
             }
         }),
     )
@@ -362,12 +300,12 @@ where
 fn handshake_service_factory2<Io, St, C>(
     factory: C,
     max_size: u32,
-    handshake_timeout: u16,
+    handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 ) -> impl ServiceFactory<
     Config = (),
     Request = (Io, State),
-    Response = (Io, State, Rc<MqttShared>, Session<St>, u16),
+    Response = (Io, State, Rc<MqttShared>, Session<St>, Seconds),
     Error = MqttError<C::Error>,
     InitError = C::InitError,
 >
@@ -376,16 +314,16 @@ where
     C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>,
     C::Error: fmt::Debug,
 {
-    ntex::apply(
-        Timeout::new(Duration::from_millis(handshake_timeout as u64)),
-        ntex::fn_factory(move || {
+    ntex::service::apply(
+        Timeout::new(Millis::from(handshake_timeout)),
+        ntex::service::fn_factory(move || {
             let pool = pool.clone();
             let fut = factory.new_service(());
             async move {
                 let service = fut.await?;
                 let pool = pool.clone();
                 let service = Rc::new(service.map_err(MqttError::Service));
-                Ok(ntex::apply_fn(service, move |(io, state), service| {
+                Ok(ntex::service::apply_fn(service, move |(io, state), service| {
                     handshake(io, Some(state), service.clone(), max_size, pool.clone())
                 }))
             }
@@ -403,7 +341,7 @@ async fn handshake<Io, S, St, E>(
     service: S,
     max_size: u32,
     pool: Rc<MqttSinkPool>,
-) -> Result<(Io, State, Rc<MqttShared>, Session<St>, u16), S::Error>
+) -> Result<(Io, State, Rc<MqttShared>, Session<St>, Seconds), S::Error>
 where
     Io: AsyncRead + AsyncWrite + Unpin,
     S: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>, Error = MqttError<E>>,
@@ -481,12 +419,10 @@ where
     }
 }
 
-type ResponseItem<U> = Option<<U as Encoder>::Item>;
-
 pub(crate) struct ServerSelector<St, C, T, Io, F, R> {
     connect: C,
     handler: Rc<T>,
-    disconnect_timeout: u16,
+    disconnect_timeout: Seconds,
     time: Timer,
     check: Rc<F>,
     max_size: u32,
@@ -496,7 +432,7 @@ pub(crate) struct ServerSelector<St, C, T, Io, F, R> {
 impl<St, C, T, Io, F, R> ServiceFactory for ServerSelector<St, C, T, Io, F, R>
 where
     Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    F: Fn(&mqtt::Connect) -> R + 'static,
+    F: Fn(&Handshake<Io>) -> R + 'static,
     R: Future<Output = Result<bool, C::Error>>,
     C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
         + 'static,
@@ -504,7 +440,7 @@ where
     T: ServiceFactory<
             Config = Session<St>,
             Request = DispatchItem<Rc<MqttShared>>,
-            Response = ResponseItem<Rc<MqttShared>>,
+            Response = Option<mqtt::Packet>,
             Error = MqttError<C::Error>,
             InitError = MqttError<C::Error>,
         > + 'static,
@@ -544,7 +480,7 @@ pub(crate) struct ServerSelectorImpl<St, C, T, Io, F, R> {
     check: Rc<F>,
     connect: Rc<C>,
     handler: Rc<T>,
-    disconnect_timeout: u16,
+    disconnect_timeout: Seconds,
     time: Timer,
     max_size: u32,
     _t: PhantomData<(St, Io, R)>,
@@ -553,14 +489,14 @@ pub(crate) struct ServerSelectorImpl<St, C, T, Io, F, R> {
 impl<St, C, T, Io, F, R> Service for ServerSelectorImpl<St, C, T, Io, F, R>
 where
     Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    F: Fn(&mqtt::Connect) -> R + 'static,
+    F: Fn(&Handshake<Io>) -> R + 'static,
     R: Future<Output = Result<bool, C::Error>>,
     C: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>> + 'static,
     C::Error: fmt::Debug,
     T: ServiceFactory<
             Config = Session<St>,
             Request = DispatchItem<Rc<MqttShared>>,
-            Response = ResponseItem<Rc<MqttShared>>,
+            Response = Option<mqtt::Packet>,
             Error = MqttError<C::Error>,
             InitError = MqttError<C::Error>,
         > + 'static,
@@ -592,24 +528,24 @@ where
         let max_size = self.max_size;
 
         Box::pin(async move {
-            let (pkt, io, state, shared, mut delay) = req;
+            let (hnd, state, mut delay) = req;
 
             let result = if let Some(ref mut delay) = delay {
-                let fut = (&*check)(&pkt);
+                let fut = (&*check)(&hnd);
                 match crate::utils::select(fut, delay).await {
                     Either::Left(res) => res,
                     Either::Right(_) => return Err(MqttError::HandshakeTimeout),
                 }
             } else {
-                (&*check)(&pkt).await
+                (&*check)(&hnd).await
             };
 
             if !result.map_err(MqttError::Service)? {
-                Ok(Either::Left((pkt, io, state, shared, delay)))
+                Ok(Either::Left((hnd, state, delay)))
             } else {
                 // authenticate mqtt connection
                 let mut ack = if let Some(ref mut delay) = delay {
-                    let fut = connect.call(Handshake::new(pkt, io, shared));
+                    let fut = connect.call(hnd);
                     match crate::utils::select(fut, delay).await {
                         Either::Left(res) => res.map_err(|e| {
                             log::trace!("Connection handshake failed: {:?}", e);
@@ -618,7 +554,7 @@ where
                         Either::Right(_) => return Err(MqttError::HandshakeTimeout),
                     }
                 } else {
-                    connect.call(Handshake::new(pkt, io, shared)).await.map_err(|e| {
+                    connect.call(hnd).await.map_err(|e| {
                         log::trace!("Connection handshake failed: {:?}", e);
                         MqttError::Service(e)
                     })?
@@ -653,7 +589,7 @@ where
                             handler,
                             time,
                         )
-                        .keepalive_timeout(ack.keepalive as u16)
+                        .keepalive_timeout(ack.keepalive)
                         .disconnect_timeout(timeout)
                         .await?;
                         Ok(Either::Right(()))
